@@ -1,30 +1,31 @@
-;;; skk-rdbms.el --- SKK Relational Data Base Management System.
-;; Copyright (C) 1998, 2000 Mikio Nakajima <minakaji@osaka.email.ne.jp>
+;;; skk-rdbms.el --- SKK Relational Data Base Management System. -*- coding: iso-2022-jp -*-
+;; Copyright (C) 1998, 2000, 2003 NAKAJIMA Mikio <minakaji@namazu.org>
 
-;; Author: Mikio Nakajima <minakaji@osaka.email.ne.jp>
-;; Maintainer: Mikio Nakajima <minakaji@osaka.email.ne.jp>
-;; Version: $Id: skk-rdbms.el,v 1.4 2000/12/13 10:38:35 minakaji Exp $
+;; Author: NAKAJIMA Mikio <minakaji@namazu.org>
+;; Maintainer: NAKAJIMA Mikio <minakaji@namazu.org>
+;; Version: $Id: skk-rdbms.el,v 1.10 2007/04/22 02:38:28 skk-cvs Exp $
 ;; Keywords: japanese, rdbms
-;; Last Modified: $Date: 2000/12/13 10:38:35 $
+;; Last Modified: $Date: 2007/04/22 02:38:28 $
 
-;; This file is not part of SKK yet.
+;; This file is not part of Daredevil SKK yet.
 
-;; SKK is free software; you can redistribute it and/or modify
-;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either versions 2, or (at your option)
-;; any later version.
+;; Daredevil SKK is free software; you can redistribute it and/or
+;; modify it under the terms of the GNU General Public License as
+;; published by the Free Software Foundation; either version 2, or
+;; (at your option) any later version.
 
-;; SKK is distributed in the hope that it will be useful
+;; Daredevil SKK is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;; GNU General Public License for more details.
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+;; General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with SKK, see the file COPYING.  If not, write to the Free
-;; Software Foundation Inc., 59 Temple Place - Suite 330, Boston,
-;; MA 02111-1307, USA.
+;; along with Daredevil SKK, see the file COPYING.  If not, write to
+;; the Free Software Foundation Inc., 51 Franklin Street, Fifth Floor,
+;; Boston, MA 02110-1301, USA.
 
 ;;; Commentary:
+
 ;; Currently this program only supports PostgreSQL, but may work with
 ;; other rdbms (such as mSQL or mySQL) by slightly changes.
 
@@ -32,12 +33,11 @@
 ;; $B$rAw$k!"$H$$$&ItJ,$O!"(BEmacs Calc $B$N(B Gnuplot $B%$%s%?!<%U%'%$%9ItJ,$G(B
 ;; $B$"$k(B calc-graph.el $B$r;29M$K$7$?!#(B
 
-;; Following people contributed modifications to skk-rdbms.el (Alphabetical order):
-
-;;; Change log:
-
 ;;; Code:
-(eval-when-compile (require 'skk-macs) (require 'skk-vars))
+
+(eval-when-compile
+  (require 'skk-macs)
+  (require 'skk-vars))
 
 ;; User variables.
 (defvar skk-rdbms-shell "psql"
@@ -85,7 +85,7 @@ skk-rdbms-shell $B$N%P!<%8%g%s$K$h$C$F0[$J$kF0$-$r$7$?$$>l9g$K;HMQ$9$k!#(B")
 
 CREATE TABLE YOUR-LOGIN-NAME_private_jisyo \(
 	okuriari int2 NOT NULL,
-	yomi varchar\(50\) NOT NULL, -- longest entry of yomi in SKK-JISYO.L is 
+	yomi varchar\(50\) NOT NULL, -- longest entry of yomi in SKK-JISYO.L is
 				   -- '$B$[$/$j$/$;$s$?$s$+$,$/$.$8$e$D$@$$$,$/$$$s$@$$$,$/(B'
 	kanji text NOT NULL,
 	okurigana varchar\(4\),
@@ -164,7 +164,7 @@ CREATE TABLE busyu_base \(
 	yomi1 varchar\(16\) NOT NULL,
 	yomi2 varchar\(16\),
 	yomi3 varchar\(16\),
-	yomi4 varchar\(16\) 
+	yomi4 varchar\(16\)
 \);")
 
 (defvar skk-rdbms-busyu-data-table "busyu_data"
@@ -222,7 +222,7 @@ CREATE TABLE hinsi_data \(
   (`
    (concat
     (format
-     "DELETE FROM %s WHERE kanji = '%s' AND yomi = '%s' AND okuriari = %s " 
+     "DELETE FROM %s WHERE kanji = '%s' AND yomi = '%s' AND okuriari = %s "
 	    skk-rdbms-private-jisyo-table (, word) skk-henkan-key
 	    (if skk-okuri-char 1 0))
     (if (and skk-henkan-okuri-strictly skk-henkan-okurigana)
@@ -453,7 +453,7 @@ skk-rdbms-working-buffer $B$NCf$G%3!<%k$5$l$k!#(B")
 (setq skk-previous-completion-function 'skk-rdbms-previous-completion)
 (setq skk-okuri-search-function 'skk-rdbms-okuri-search)
 (setq skk-public-jisyo-to-be-searched-function 'skk-rdbms-public-jisyo-to-be-searched)
- 
+
 (defun skk-rdbms-init ()
   (or (skk-rdbms-process-alive)
       (let ((process-connection-type t)
@@ -686,14 +686,14 @@ C-u M-x skk-rdbms-restore-private-jisyo $B$9$k$H3NG'$J$7$KI|85$9$k!#(B"
 	    shell-file-name
 	    nil (list nil skk-rdbms-dump-error) t
 	    "-c" (format "%s %s < %s" skk-rdbms-shell ; PostgreSQL only command.
-			 skk-rdbms-shell-args 
+			 skk-rdbms-shell-args
 			 (expand-file-name skk-rdbms-private-jisyo-dump)))
 	   0)
 	(skk-message "$B%@%s%W%U%!%$%k$+$i8D?M<-=q%G!<%?%Y!<%9$rI|85$7$F$$$^$9(B...$B40N;(B!"
 		     "Restoring private jisyo database from dump file...done!")
       (skk-error "$B%@%s%W%U%!%$%k$+$i$N8D?M<-=q%G!<%?%Y!<%9$NI|85$K<:GT$7$^$7$?(B"
 		 "Failed to restore private jisyo database from dump file"))))
-      
+
 (defun skk-rdbms-count-jisyo-candidates (table)
   (if (interactive-p)
       (message "Counting jisyo candidates..."))
@@ -834,7 +834,7 @@ C-u M-x skk-rdbms-restore-private-jisyo $B$9$k$H3NG'$J$7$KI|85$9$k!#(B"
 	     skk-okuri-char
 	     (eq ?z (car (memq (string-to-char skk-okuri-char) '(?s ?z))))))
 	   ((and okurigana
-		 (string= "$B$:(B" (char-to-string (skk-str-ref okurigana 0)))))))
+		 (string= "$B$:(B" (char-to-string (aref okurigana 0)))))))
 	 (skk-henkan-key
 	  (cond
 	   (skk-okuri-char
@@ -892,7 +892,7 @@ C-u M-x skk-rdbms-restore-private-jisyo $B$9$k$H3NG'$J$7$KI|85$9$k!#(B"
 		      (if skk-japanese-message-and-error "$B4A;z(B: " "Kanji: "))))
   (if (interactive-p)
       (message (skk-rdbms-kanji-to-stroke kanji))
-   (let (v) 
+   (let (v)
      (add-hook 'minibuffer-setup-hook 'skk-j-mode-on)
      (add-hook
       'minibuffer-setup-hook
@@ -907,7 +907,7 @@ C-u M-x skk-rdbms-restore-private-jisyo $B$9$k$H3NG'$J$7$KI|85$9$k!#(B"
        (if skk-japanese-message-and-error
 	   (concat v " $B2h(B")
 	 (concat v " stroke" (if (not (string= v "1")) "s")))))))
-  
+
 ;; skk-rdbms-save-jisyo $B$NCf$G(B VACUUM $B$7$?8e$K%W%m%;%9$r;&$9!#(B
 (add-hook 'skk-before-kill-emacs-hook 'skk-rdbms-kill 'append)
 
