@@ -1,6 +1,7 @@
-;;; skk-kanagaki-util.el --- SKK $B$N2>L>F~NO%5%]!<%H$N$?$a$NF;6qH"(B
+;;; skk-kanagaki-util.el --- SKK $B$N2>L>F~NO%5%]!<%H$N$?$a$NF;6qH"(B -*- coding: iso-2022-jp -*-
 
-;; Copyright (C) 2000 Tetsuo Tsukamoto <czkmt@remus.dti.ne.jp>
+;; Copyright (C) 2000, 2001, 2002, 2003, 2004
+;;   Tetsuo Tsukamoto <czkmt@remus.dti.ne.jp>
 
 ;; Author: Tetsuo Tsukamoto <czkmt@remus.dti.ne.jp>
 ;; Keywords: hardware, japanese, mule, input method
@@ -19,8 +20,8 @@
 
 ;; You should have received a copy of the GNU General Public License
 ;; along with Daredevil SKK, see the file COPYING.  If not, write to
-;; the Free Software Foundation Inc., 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
+;; the Free Software Foundation Inc., 51 Franklin Street, Fifth Floor,
+;; Boston, MA 02110-1301, USA.
 
 ;;; Commentary:
 
@@ -62,15 +63,14 @@
 
 ;;;###autoload
 (defmacro skk-kanagaki-help-1 (bufname title list)
-  (`
-   (let ((buf (get-buffer-create (, bufname))))
+  `(let ((buf (get-buffer-create ,bufname)))
      (save-excursion
        (set-buffer buf)
        (setq buffer-read-only nil)
        (erase-buffer)
        (insert
 	(concat
-	 (format "%s\n\n" (, title))
+	 (format "%s\n\n" ,title)
 	 (mapconcat
 	  #'(lambda (cons)
 	      (cond
@@ -82,7 +82,7 @@
 	       (t
 		(format "%s $B!D(B %s\n" (car cons) (cdr cons)))))
 	  ;;
-	  (delq nil (, list)) "")))
+	  (delq nil ,list) "")))
        ;;
        (setq buffer-read-only t)
        (set-buffer-modified-p nil)
@@ -90,46 +90,7 @@
        (help-mode))
      (let ((standard-output buf))
        (print-help-return-message))
-     (display-buffer buf))))
-
-;;;###autoload
-(put 'skk-kanagaki-call-xmodmap 'lisp-indent-function 1)
-
-;;;###autoload
-(defmacro skk-kanagaki-call-xmodmap (string &rest form)
-  ;; STRING $B$NFbMF$r(B xmodmap $B$KEO$9!#@.8y$7$?$i(B FORM $B$r<B9T$9$k!#(B
-  (list
-   'let '((x (eq window-system 'x))
-	  (prog (exec-installed-p "xmodmap"))
-	  (tmp (make-temp-file "kanagaki")))
-   (list
-    'cond
-    (list
-     (list 'and 'x 'prog
-	   '(message "xmodmap $B$r8F$s$G$$$^$9(B...")
-	   (list
-	    'save-excursion
-	    '(set-buffer (get-buffer-create " *kanagaki*"))
-	    '(erase-buffer)
-	    (list 'insert string)
-	    '(write-region (point-min) (point-max) tmp)
-	    '(eq 0 (call-process prog nil nil nil tmp))))
-     ;;
-     (` (progn
-	  (,@ form)))
-     '(delete-file tmp)
-     '(message "xmodmap $B$r8F$s$G$$$^$9(B...$B40N;(B"))
-    '(t
-      (message "xmodmap $B$N8F$S=P$7$K<:GT$7$^$7$?(B")))))
-
-;;;###autoload
-(defmacro skk-kanagaki-make-string (n str)
-  (case skk-emacs-type
-   ((xemacs mule4 mule5)
-    (` (make-string (, n) (string-to-char (, str)))))
-   (t
-    (` (mapconcat 'identity
-		  (make-vector (, n) (, str)) "")))))
+     (display-buffer buf)))
 
 ;;;###autoload
 (defun skk-nicola-visit-nicola-website ()
@@ -185,22 +146,75 @@
   (interactive "*p")
   (let ((list skk-kanagaki-dakuten-alist)
 	(pt1 (point))
-	char1 char2)
+	(henkan-on (and skk-isearch-switch
+			(with-current-buffer
+			    (get-buffer-create skk-isearch-working-buffer)
+			  (eq skk-henkan-mode 'on))))
+	char1 char2 str)
     (ignore-errors
-      (setq char1 (skk-save-point
-		   (backward-char 1)
-		   (buffer-substring-no-properties
-		    (point)
-		    pt1))))
+      (setq char1 (cond
+		   ((and skk-isearch-switch
+			 (not (skk-in-minibuffer-p)))
+		    (if henkan-on
+			(with-current-buffer skk-isearch-working-buffer
+			  (skk-save-point
+			   (backward-char 1)
+			   (buffer-substring-no-properties
+			    (point)
+			    pt1)))
+		      (substring isearch-string -1)))
+		   (t
+		    (skk-save-point
+		     (backward-char 1)
+		     (buffer-substring-no-properties
+		      (point)
+		      pt1))))))
     (cond
-     ((setq char2 (funcall (if handakuten
-			       'caddr
-			     'cadr)
-			   (assoc char1 list)))
-      (delete-char -1)
-      (skk-insert-str char2))
+     ((setq char2 (nth (if handakuten 2 1) (assoc char1 list)))
+      (cond
+       ((and skk-isearch-switch
+	     (not (skk-in-minibuffer-p)))
+	(if henkan-on
+	    (with-current-buffer skk-isearch-working-buffer
+	      (delete-char -1)
+	      (skk-insert-str char2))
+	  (setq str isearch-string)
+	  (while (string= str (if (vectorp (car isearch-cmds))
+				  (aref (car isearch-cmds) 0)
+				(caar isearch-cmds)))
+	    (with-current-buffer skk-isearch-current-buffer
+	      (skk-isearch-delete-char arg)))
+	  (setq isearch-string (concat (if (vectorp (car isearch-cmds))
+					   (aref (car isearch-cmds) 0)
+					 (caar isearch-cmds))
+				       char2)
+		isearch-message (concat
+				 (skk-isearch-mode-string)
+				 (mapconcat
+				  #'isearch-text-char-description
+				  isearch-string "")))
+	  (put 'isearch-barrier 'skk-kanagaki t)
+	  (skk-unread-event (character-to-event
+			     (aref (where-is-internal
+				    (if isearch-forward 'isearch-repeat-forward
+				      'isearch-repeat-backward)
+				    isearch-mode-map t)
+				   0)))))
+       (t
+	(delete-char -1)
+	(skk-insert-str char2))))
      (t
-      (skk-insert-str "$B!+(B")))))
+      (skk-insert-str (if handakuten
+			  "$B!,(B"
+			"$B!+(B"))))))
+
+(defadvice isearch-repeat (around skk-kanagaki-workaround activate)
+  (cond ((get 'isearch-barrier 'skk-kanagaki)
+	 (goto-char isearch-barrier)
+	 ad-do-it
+	 (put 'isearch-barrier 'skk-kanagaki nil))
+	(t
+	 ad-do-it)))
 
 ;;;###autoload
 (defun skk-kanagaki-handakuten (&optional arg)
@@ -219,16 +233,6 @@
   ;;
   ;; $B$H$$$&$U$&$K9M$($F$$$k!#(B
   (interactive "*p")
-  ;; skk-dcomp $BMxMQ;~$N:]$N;H$$>!<j$r$h$/$9$k!#(B
-  (when (and (featurep 'skk-dcomp)
-	     (eq skk-henkan-mode 'on)
-	     (markerp skk-dcomp-end-point)
-	     (marker-position skk-dcomp-end-point))
-    (delete-region
-     skk-dcomp-end-point
-     (if (< (point) (marker-position skk-dcomp-start-point))
-	 skk-dcomp-start-point
-       (point))))
   ;;
   (cond
    ((eq skk-henkan-mode 'active)
@@ -245,7 +249,10 @@
     (with-current-buffer skk-isearch-current-buffer
       (skk-isearch-delete-char arg)))
    (t
-    (delete-backward-char arg))))
+    (delete-backward-char arg)))
+  ;;
+  (when skk-dcomp-activate
+    (skk-dcomp-after-delete-backward-char)))
 
 ;;;###autoload
 (defun skk-kanagaki-esc (&optional arg)
